/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

/*
 * $Revision: 1.12 $
 *
 * DevInfo_t definetions
 */

#ifndef __mcdevinfo_h__
#define __mcdevinfo_h__ 

/*
 * Main Device information
 *
 * Used after device info has been obtained 
 */
struct _DevInfo {
    char			*Name;		/* Name (e.g. cgtwo0) */
    char			*Driver;	/* Driver Name (e.g. cgtwo) */
    char		       **Aliases;	/* Alias Names */
    char			*AltName;	/* Alt name */
    char		       **Files;		/* Device files */
    int				 Type;		/* Device type (eg DT_TAPE) */
    int				 ClassType;	/* Class type (eg SCSI,IPI) */
    char			*Vendor;	/* Hardware Vendor */
    char			*Model;		/* Model */
    char			*ModelDesc;	/* eg SCSI, 4.0GB, etc. */
    char			*Capacity;	/* Capacity of device */
    char			*Speed;		/* Speed of device */
    char			*Part;		/* Part Number */
    char			*Serial;	/* Serial Number */
    Ident_t			*Ident;		/* Unique Identifier */
    char			*Revision;	/* Revision Info */
    DevDesc_t			*DescList;	/* Device Description */
    int				 Unit;		/* Unit # unique to Type */
    int				 Bus;		/* Bus # unique to Type */
    int				 DevNum;	/* Device number on Bus */
    int				 Port;		/* Port # on Bus */
    int				 NodeID;	/* ID of this node */
    void			*DevSpec;	/* Device specific info */
    char			*MasterName;	/* Name of master */
    struct _DevInfo		*Master;	/* Device controller */
    struct _DevInfo		*Slaves;	/* Devices on this device */
    struct _DevInfo		*Next;		/* Pointer to next device */
    /* Internal use only */
    void			*OSdata;	/* Data from OS */
    /* Obsolete */
    int				 Addr;		/* Address */
    int				 Prio;		/* Priority */
    int				 Vec;		/* Vector */
};
typedef struct _DevInfo DevInfo_t;

/*
 * Device types (DevInfo_t.Type)
 */
#define DT_NONE			1		/* Sort of ignore */
#define DTN_NONE		"none"
#define DT_PSEUDO		2		/* Pseudo Device */
#define DTN_PSEUDO		"pseudo"
#define DT_DRIVER		3		/* Device Driver */
#define DTN_DRIVER		"driver"
#define DT_GENERIC		4		/* Generic Device */
#define DTN_GENERIC		"generic"
#define DT_DEVICE		5		/* Real Device */
#define DTN_DEVICE		"device"
#define DT_DISKDRIVE		6		/* Disk Drive */
#define DTN_DISKDRIVE		"diskdrive"
#define DT_DISKCTLR		7		/* Disk Controller */
#define DTN_DISKCTLR		"diskctlr"
#define DTN_MSD			"msd"
#define DT_TAPEDRIVE		8		/* Tape Drive */
#define DTN_TAPEDRIVE		"tapedrive"
#define DT_TAPECTLR		9		/* Tape Controller */
#define DTN_TAPECTLR		"tapectlr"
#define DT_FRAMEBUFFER		10		/* Frame Buffer */
#define DT_FB			DT_FRAMEBUFFER
#define DTN_FRAMEBUFFER		"framebuffer"
#define DTN_FB			"fb"
#define DTN_VID			"vid"
#define DT_GFXACCEL		11		/* Graphics Accelerator */
#define DTN_GFXACCEL		"gfxaccel"	/* Not the same as DT_FRAME* */
#define DT_NETIF		12		/* Network Interface */
#define DTN_NETIF		"netif"
#define DTN_NET			"net"
#define DT_BUS			13		/* System Bus */
#define DTN_BUS			"bus"
#define DT_BRIDGE		14		/* System Bus Bridge */
#define DTN_BRIDGE		"bridge"
#define DT_CONTROLLER		15		/* Generic Controller */
#define DT_CTLR			DT_CONTROLLER
#define DTN_CONTROLLER		"ctlr"
#define DTN_ADAPTER		"adapter"
#define DT_PROCESSOR		16		/* Processor */
#define DTN_PROCESSOR		"processor"
#define DT_CPU			16		/* for backwards compat */
#define DTN_CPU			"cpu"
#define DT_MEMORY		17		/* Memory */
#define DTN_MEMORY		"memory"
#define DTN_MEM			"mem"
#define DT_KEYBOARD		18		/* Keyboard */
#define DTN_KEYBOARD		"keyboard"
#define DTN_KEY			"key"
#define DT_CD			19		/* CD-* */
#define DTN_CDROM		"cdrom"
#define DT_SERIAL		20		/* Serial */
#define DTN_SERIAL		"serial"
#define DTN_COM			"com"
#define DT_PARALLEL		21		/* Parallel */
#define DTN_PARALLEL		"parallel"
#define DTN_PRT			"prt"
#define DT_CARD			22		/* Generic Card */
#define DTN_CARD		"card"
#define DT_AUDIO		23		/* Audio */
#define DTN_AUDIO		"audio"
#define DT_MFC			24		/* Multi Function Card */
#define DTN_MFC			"mfc"
#define DT_POINTER		25		/* Pointer (mouse, etc) */
#define DTN_POINTER		"pointer"
#define DTN_PTR			"ptr"
#define DTN_MOUSE		"mouse"
#define DT_MONITOR		26		/* Video Monitor */
#define DTN_MONITOR		"monitor"
#define DT_FLOPPY		27		/* Floppy Disk Drive */
#define DTN_FLOPPY		"floppy"
#define DT_FLOPPYCTLR		28		/* Floppy Disk Controller */
#define DTN_FLOPPYCTLR		"floppyctlr"
#define DT_PRINTER		29		/* Printer */
#define DTN_PRINTER		"printer"
#define DT_WORM			30		/* WORM */
#define DTN_WORM		"WORM"
#define DT_SCANNER		31		/* Scanner */
#define DTN_SCANNER		"scanner"
#define DT_CONSOLE		32		/* Console */
#define DTN_CONSOLE		"console"
#define DT_DVD			33		/* DVD-* */
#define DTN_DVD			"dvd"
#define DT_BIOS			34		/* system BIOS */
#define DTN_BIOS		"bios"
#define DT_SYSBOARD		35		/* system board */
#define DTN_SYSBOARD		"sysboard"
#define DT_ENCLOSURE		36		/* enclosure/chassis */
#define DTN_ENCLOSURE		"enclosure"
#define DT_MEMCTL		37		/* memory controller */
#define DTN_MEMCTL		"memctl"

/*
 * Backwards compatibility
 */
#define DT_CDROM		DT_CD		/* CD-ROM */
#define DTN_CD			"cd"

#endif	/* __mcdevinfo_h__ */
