/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

/*
 * $Revision: 1.4 $
 *
 * DiskDrive definetions
 */

#ifndef __mcdiskdrive_h__
#define __mcdiskdrive_h__ 

/*
 * Values for DiskDrive_t->DataType
 */
#define DK_DTYPE_HW	"HW"		/* Hardware */
#define DK_DTYPE_OS	"OS"		/* OS */

/*
 * Disk Drive specific data
 */
struct _DiskDrive {
    char		*DataType;	/* HW, OS */
    char		*Label;		/* Disk label */
    int			 Unit;		/* Unit number */
    int			 Slave;		/* Slave number */
    /* Cylinders */
    u_long		 DataCyl;	/* # (usable) data cylinders */
    u_long		 PhyCyl;	/* # physical cylinders */
    u_long		 AltCyl;	/* # alternate cylinders */
    u_long		 CylSkew;	/* Cylinder Skew */
    u_long		 APC;		/* Alternates / Cyl (SCSI) */
    /* Tracks (Heads) */
    u_long		 Tracks;	/* # Tracks */
    u_long		 AltTracksPerZone;/* # Alt Tracks / zone */
    u_long		 AltTracksPerVol;/* # alternate Tracks / volume */
    u_long		 TrackSkew;	/* Track Skew */
    /* Sectors */
    u_long		 Sect;		/* # usable sectors / Track */
    int			 SecSize;	/* Size of Sector (bytes) */
    u_long		 PhySect;	/* # physical sectors / Track */
    u_long		 AltSectPerZone;/* # Alternate Sectors / Zone */
    u_long		 SectGap;	/* Gap length between sectors */
    /* Floppy Related */
    u_long		 StepsPerTrack;	/* Steps per track */
    /* General Misc */
    u_long		 RPM;		/* Revolutions Per Minute */
    u_long		 IntrLv;	/* Interleave factor */
    u_long		 PROMRev;	/* PROM Revision */
    /* SysInfo Misc */
    float		 Size;		/* Size of disk (MB) */
    int			 Flags;		/* Info flags */
    DevInfo_t		*Ctlr;		/* Controller disk is on */
    struct _DiskPart	*DiskPart;	/* Partition information */
};
typedef struct _DiskDrive DiskDrive_t;

/*
 * DiskDriveData_t contains multiple DiskDrive_t's because the actual
 * hardware data may differ from what the OS uses.  For instance in
 * x86 UNIX land, the OS may be installed inside of a DOS partition.
 * In this case, the OS may think the disk is "600MB" when in fact it's
 * actually installed in a 600MB partition on a 4GB disk.
 */
typedef struct {
    DiskDrive_t		*HWdata;	/* The actual hardware Drive data */
    DiskDrive_t		*OSdata;	/* The OS's view of the Drive */
} DiskDriveData_t;
    
/*
 * Disk Flags (DiskDrive_t.Flags)
 */
#define DF_HEXUNIT	0x01			/* Unit is prt 3 hex digits */

#endif	/* __mcdiskdrive_h__ */
