/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.6 $";
#endif

/*
 * Get Kernel Bit Size (32/64/...)
 */

#include "defs.h"

/*
 * Get Kernel Bit Size based on CpuType
 */
extern Opaque_t GetKernBitsCpuType()
{
    MCSIquery_t			Query;
    static char			Buff[8];
    char		       *cp;

    if (Buff[0])
	return(Buff);

    (void) memset(&Query, CNULL, sizeof(Query));
    Query.Cmd = MCSI_CPUTYPE;
    if (mcSysInfo(&Query) != 0) {
	return (char *) NULL;
    }

    cp = (char *) Query.Out;
    SImsg(SIM_DBG, "GetKernBitsCpuType: cputype=<%s>", PRTS(cp));

    /*
     * Solaris returns "sparcv9..." when run in 64-bit mode or
     * "sparcv8..." when in 32-bit mode.
     */
    if (EQN(cp, "sparcv9", 7))
	snprintf(Buff, sizeof(Buff), "64");
    else if (EQN(cp, "sparc", 5))
	snprintf(Buff, sizeof(Buff), "32");
    /*
     * HP PA-RISC
     *
     * The value of cp should look something like "PA7100LC" or "PA8700".
     * The PA7xxx series is 32-bit and PA8xxx and later is 64-bit.
     */
    else if (EQN(cp, "pa7", 3))
	snprintf(Buff, sizeof(Buff), "32");
    else if (EQN(cp, "pa8", 3))
	snprintf(Buff, sizeof(Buff), "64");

    return( (Buff[0]) ? Buff : (char *) NULL );
}

/*
 * Get Kernel Bit Size based on kernel architecture
 */
extern Opaque_t GetKernBitsKernArch()
{
    MCSIquery_t			Query;
    static char			Buff[8];
    char		       *cp;

    if (Buff[0])
	return(Buff);

    (void) memset(&Query, CNULL, sizeof(Query));
    Query.Cmd = MCSI_KERNARCH;
    if (mcSysInfo(&Query) != 0) {
	return (char *) NULL;
    }

    cp = (char *) Query.Out;
    SImsg(SIM_DBG, "GetKernBitsKernArch: karch=<%s>", PRTS(cp));

    /*
     * x86 kernel architecture values
     */
    if (EQ(cp, "i386") || EQ(cp, "i486") || EQ(cp, "i586") || EQ(cp, "i686"))
	snprintf(Buff, sizeof(Buff), "32");

    return( (Buff[0]) ? Buff : (char *) NULL );
}

/*
 * Get Kernel Bit Size based on OSname
 */
extern Opaque_t GetKernBitsOsName()
{
    MCSIquery_t			Query;
    static char			Buff[8];
    char		       *cp;

    if (Buff[0])
	return(Buff);

    (void) memset(&Query, CNULL, sizeof(Query));
    Query.Cmd = MCSI_OSNAME;
    if (mcSysInfo(&Query) != 0) {
	return (char *) NULL;
    }

    cp = (char *) Query.Out;
    SImsg(SIM_DBG, "GetKernBitsOsName: osname=<%s>", PRTS(cp));

    if (EQ(cp, "irix"))
	snprintf(Buff, sizeof(Buff), "32");
    else if (EQ(cp, "irix64"))
	snprintf(Buff, sizeof(Buff), "64");

    return( (Buff[0]) ? Buff : (char *) NULL );
}

/*
 * Get Kernel Bit Size
 */
extern int GetKernBits(Query)
     MCSIquery_t	      *Query;
{
    extern PSI_t	       GetKernBitsPSI[];
    char		      *Str;

    if (Query->Op == MCSIOP_CREATE) {
	if (Str = PSIquery(PsiSwitch.GetKernBits)) {
	    Query->Out = (Opaque_t) strdup(Str);
	    Query->OutSize = strlen(Str);
	    return 0;
	}
    } else if (Query->Op == MCSIOP_DESTROY) {
	if (Query->Out && Query->OutSize)
	    (void) free(Query->Out);
	return 0;
    }

    return -1;
}
