/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.11 $";
#endif

/*

 * Get LibC info
 */

#include "defs.h"

/*
 * Path name to any program which is dynamically linked
 */
#if	!defined(LDD_TEST_PROGRAM)
#define LDD_TEST_PROGRAM	"/bin/sh"
#endif

static char *LddPaths[] = { "/usr/ccs/bin/ldd", "/usr/bin/ldd", "/bin/ldd",
			    "/usr/contrib/bin/ldd",
			    0 };
static char *GrepPaths[] = { "/bin/grep", "/usr/bin/grep", 0 };
static char *AwkPaths[] = { "/bin/awk", "/usr/bin/awk", 0 };
static char *StringsPaths[] = { "/usr/ccs/bin/strings", "/usr/bin/strings", 
				"/bin/strings", 0 };

/*
 * Get the version string from a generic shared libc system
 */
extern int GetLibcShared(LibcName, LibcVersion)
     char                    **LibcName;
     char                    **LibcVersion;
{
    static char		       *SaveLibcVersion;
    static char		       *SaveLibcName;
    static char			Cmd[1024];
    char		       *Cmds[2];
    char		       *LibcPath;
    char		       *cp;
    char		       *Ldd;
    char		       *Grep;
    char		       *Awk;
    char		       *Strings;
    char		       *VerStr;
    int				Argc;
    char		      **Argv;

    SImsg(SIM_DBG, "GetLibcShared called");

    if (SaveLibcName && SaveLibcVersion) {
	if (LibcName)
	    *LibcName = SaveLibcName;
	if (LibcVersion)
	    *LibcVersion = SaveLibcVersion;
	return 0;
    }

    if (!(Ldd = FindCommand(LddPaths))) {
	SImsg(SIM_DBG, "GetLibcShared: Cannot find ldd command.");
	return -1;
    }
    if (!(Grep = FindCommand(GrepPaths))) {
	SImsg(SIM_DBG, "GetLibcShared: Cannot find grep command.");
	return -1;
    }
    if (!(Awk = FindCommand(AwkPaths))) {
	SImsg(SIM_DBG, "GetLibcShared: Cannot find awk command.");
	return -1;
    }
    if (!(Strings = FindCommand(StringsPaths))) {
	SImsg(SIM_DBG, "GetLibcShared: Cannot find strings command.");
	return -1;
    }

    Cmds[1] = NULL;
    /*
     * First determine the pathname of libc.
     * Ugly, but it does work
     */
    snprintf(Cmd, sizeof(Cmd), 
	     "(%s %s | %s libc. | %s '{print $3}') 2> /dev/null",
	     Ldd, LDD_TEST_PROGRAM, Grep, Awk);
    Cmds[0] = Cmd;
    cp = RunCmds(Cmds, 0);
    if (!cp || !*cp) {
	SImsg(SIM_DBG, "GetLibcShared(): Cannot determine Libc pathname.");
	return -1;
    }
    LibcPath = strdup(cp);

    /*
     * Now look for a GNU version string
     */
    (void) snprintf(Cmd, sizeof(Cmd), 
		    "(%s %s | %s -i 'c library') 2> /dev/null",
		    Strings, LibcPath, Grep);
    Cmds[0] = Cmd;
    if (cp = RunCmds(Cmds, 0)) {
	/* This is a GNU libc */
	SaveLibcName = "glibc";
	SImsg(SIM_DBG, "GetLibcShared(): VersionStr=<%s>", cp);

	VerStr = strdup(cp);

	if (cp = strchr(VerStr, ','))
	    *cp = ' ';
	Argc = StrToArgv(VerStr, " ", &Argv, NULL, 0);
	if (Argc >= 7)
	    SaveLibcVersion = Argv[6];
    } else {
	/* Not GNU */
	SImsg(SIM_DBG, "GetLibcShared(): No GNU version string present.");

	SaveLibcName = "libc";
	/*
	 * Get the version number assuming LibcPath looks like:
	 *    /usr/lib/libc.1
	 *    /usr/lib/libc.so.1
	 */
	if (cp = strrchr(LibcPath, '/'))
	    VerStr = cp + 1;
	else
	    VerStr = LibcPath;
	if (cp = strchr(VerStr, '.')) {
	    VerStr = cp + 1;
	    if (EQN(VerStr, "so.", 3))
		VerStr += 3;
	    if (VerStr && *VerStr)
		SaveLibcVersion = VerStr;
	}
    }

    if (LibcName)
	*LibcName = SaveLibcName;
    if (LibcVersion)
	*LibcVersion = SaveLibcVersion;

    return 0;

}

/*
 * Get the name of a shared libc
 */
extern Opaque_t GetLibcNameShared()
{
    char		       *Name = NULL;

    if (GetLibcShared(&Name, NULL) == 0)
	return Name;
    else
	return (Opaque_t) NULL;
}

/*
 * Get the version of a shared libc
 */
extern Opaque_t GetLibcVersionShared()
{
    char		       *Version = NULL;

    if (GetLibcShared(NULL, &Version) == 0)
	return Version;
    else
	return (Opaque_t) NULL;
}

/*
 * Get the name of the system's LibC
 */
extern int GetLibcName(Query)
     MCSIquery_t	      *Query;
{
    extern PSI_t	       GetLibcNamePSI[];
    char		      *Str;

    if (Query->Op == MCSIOP_CREATE) {
	if (Str = PSIquery(PsiSwitch.GetLibcName)) {
	    Query->Out = (Opaque_t) strdup(Str);
	    Query->OutSize = strlen(Str);
	    return 0;
	}
    } else if (Query->Op == MCSIOP_DESTROY) {
	if (Query->Out && Query->OutSize)
	    (void) free(Query->Out);
	return 0;
    }

    return -1;
}

/*
 * Get the version of the system's LibC
 */
extern int GetLibcVer(Query)
     MCSIquery_t	      *Query;
{
    extern PSI_t	       GetLibcVerPSI[];
    char		      *Str;

    if (Query->Op == MCSIOP_CREATE) {
	if (Str = PSIquery(PsiSwitch.GetLibcVer)) {
	    Query->Out = (Opaque_t) strdup(Str);
	    Query->OutSize = strlen(Str);
	    return 0;
	}
    } else if (Query->Op == MCSIOP_DESTROY) {
	if (Query->Out && Query->OutSize)
	    (void) free(Query->Out);
	return 0;
    }

    return -1;
}
