/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.8 $";
#endif

/*
 * Linux specific functions
 */

#include "defs.h"
#include <linux/fd.h>
#if HAVE_SETJMP_H
# include <setjmp.h>
static jmp_buf			Env;
#endif

/*
 * Onboard floppy devices
 */
static char *FloppyDevs[] = {
    "/dev/fd0", "/dev/fd1", "/dev/fd2", "/dev/fd3", NULL
};

/*
 * Called when the alarm goes off
 */
static void TimeOutHandler(SigNum)
     int			SigNum;
{
    if (SigNum == SIGALRM)
	SImsg(SIM_DBG,
	      "TimeOutHandler: A time-out has occured executing a comamnd.");
    else
	SImsg(SIM_DBG,
	      "TimeOutHandler: Cmd Received signal %s",
	      SignalName(SigNum));

#if	defined(HAVE_LONGJMP)
    longjmp(Env, SigNum);
#endif	/* HAVE_LONGJMP */
}

/*
 * Probe a Floppy device
 */
extern DevInfo_t *ProbeFloppy(ProbeData)
     ProbeData_t	       *ProbeData;
{
    DevInfo_t		       *DevInfo = NULL;
    static char			Type[512];
    int 			LongJumpVal;
    char		       *cp;

    SImsg(SIM_DBG, "%s: ProbeFloppy Name=<%s>",
	  ProbeData->DevFile, ProbeData->DevName);

#if	defined(HAVE_SETJMP)
    /*
     * Set a timeout since programs may hang
     */
    if ((LongJumpVal = setjmp(Env)) != 0) {
	/*
	 * If we reach this point then the time-out has occured
	 */
	cp = SignalName(LongJumpVal);
	SImsg(SIM_DBG, "ProbeFloppy: Probe terminated due to %s", 
	      cp);
	return (DevInfo_t *) NULL;
    }
    TimeOutStart(10, TimeOutHandler);
#endif	/* HAVE_SETJMP */

    DevInfo = ProbeData->UseDevInfo;
    if (!DevInfo)
	DevInfo = NewDevInfo(NULL);
    DevInfo->Type = DT_FLOPPY;
    if (ProbeData->DevName)	
	DevInfo->Name = strdup(ProbeData->DevName);
    if (ProbeData->DevFile)
	DevAddFile(DevInfo, strdup(ProbeData->DevFile));

#if	defined(FDGETDRVTYP)
    if (ioctl(ProbeData->FileDesc, FDGETDRVTYP, (void *) Type) == 0) {
	SImsg(SIM_DBG, "%s: ioctl FDGETDRVTYP result=<%s>",
	      ProbeData->DevFile, Type);

	/* This usually indicates the device isn't present */
	if (!Type[0] || EQ(Type, "(null)")) {
	    SImsg(SIM_DBG, "%s: Found suspect (null) floppy - ignoring.",
		  ProbeData->DevFile);
	    TimeOutEnd();
	    return (DevInfo_t *) NULL;
	}
	    
	if (EQ(Type, "H1440"))
	    DevInfo->Model = strdup("1.44MB 3.5-inch");
	else
	    DevInfo->Model = strdup(Type);
    } else {
	SImsg(SIM_DBG, "%s: ioctl FDGETDRVTYP failed: %s", 
	      ProbeData->DevFile, SYSERR);
    }
#endif

    TimeOutEnd();

    return DevInfo;
}

/*
 * Detect floppy devices
 */
extern int DetectFloppy(DevInfo_t **TreePtr, char **SearchNames)
{
    static ProbeData_t		ProbeData;
    char		      **Path;
    int				fd;
    DevInfo_t		       *DevInfo;

    for (Path = FloppyDevs; Path && *Path; ++Path) {
	SImsg(SIM_DBG, "DetectFloppy: Checking <%s>", *Path);
	/*
	 * If we can't open the device then it doesn't exist
	 */
	fd = open(*Path, O_RDONLY|O_ACCMODE);
	if (fd < 0) {
	    SImsg(SIM_DBG, "%s: Open for read failed: %s", *Path, SYSERR);
	    continue;
	}
	(void) memset(&ProbeData, CNULL, sizeof(ProbeData));
	ProbeData.DevName = BaseName(*Path);
	ProbeData.DevFile = *Path;
	ProbeData.FileDesc = fd;
	if (DevInfo = ProbeFloppy(&ProbeData))
	    AddDevice(DevInfo, TreePtr, SearchNames);
	if (fd)
	    close(fd);
    }
}
