/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.3 $";
#endif

#include "defs.h"
#include "dmi.h"

/*
 * Slot Data Bus Widths
 * Section 3.3.10.2
 */
static char *BusWidths[] = {
    "",
    "",
    "",
    "8-bit",
    "16-bit",
    "32-bit",
    "64-bit",
    "128-bit",
    NULL
};

/*
 * Slot Lengths
 * Section 3.3.10.4
 */
static char *Lengths[] = {
    "",
    "",
    "",
    "Short",
    "Long",
    NULL
};

/*
 * Decode DMI System Slot (DMI_STRUCT_SLOT)
 */
extern DevInfo_t *DmiDecodeSlot(Hdr, Data, Dmi, Query)
     DmiHeader_t	       *Hdr;
     void		       *Data;
     Dmi_t		       *Dmi;
     DmiQuery_t		       *Query;
{
    DevInfo_t		       *DevInfo;
    DevDesc_t		      **lp;
    DmiStructSlot_t	       *Slot;
    char		       *s = "Supports";
    char		       *cp;
    char		       *Design = NULL;
    char		       *Desc = NULL;
    char		       *Dev = NULL;
    static char			Buff[128];
    static int			Unit;

    Slot = (DmiStructSlot_t *) Data;

    if (cp = DmiString(Hdr, Slot->Designation))
	Design = strdup(cp);
    SImsg(SIM_DBG, 
	  "DMI SLOT: Handle=0x%04X Type=0x%02X Designation=<%s> ID=%d",
	  Hdr->Handle, Slot->Type, PRTS(Design), Slot->ID);
    SImsg(SIM_DBG, 
	  "DMI SLOT:\tsizeof(DmiStructSlot_t)=%d (0x%X) Hdr->Length=%d (0x%X)",
	  sizeof(DmiStructSlot_t), sizeof(DmiStructSlot_t), 
	  Hdr->Length, Hdr->Length);

    /*
     * If this is a specific query, then set the values now and return
     */
    if (Query && Query->Type == Hdr->Type) {
	/* Return nothing for now */
	return DevInfo;
    }

    DevInfo = NewDevInfo(NULL);
    DevInfo->Type = DT_BUS;
    lp = &(DevInfo->DescList);

    switch (Slot->Type) {
	/* Values per section 3.3.10.1 */
    case 0x00:
    case 0x01:
    case 0x02:
	break;
    case 0x03:	DevInfo->ClassType = CT_ISA;		break;
    case 0x04:	DevInfo->ClassType = CT_MCA;		break;
    case 0x05:	DevInfo->ClassType = CT_EISA;		break;
    case 0x06:	DevInfo->ClassType = CT_PCI;		break;
    case 0x07:	DevInfo->ClassType = CT_PCMCIA;		break;
    case 0x08:	DevInfo->ClassType = CT_VESA;		break;
    case 0x09:	Desc = "Proprietary";			break;
    case 0x0A:	Desc = "Processor Card";		break;
    case 0x0B:	Desc = "Proprietary Memory Card";	break;
    case 0x0C:	Desc = "I/O Riser Card";		break;
    case 0x0D:	DevInfo->ClassType = CT_NUBUS;		break;
    case 0x0E:	
	DevInfo->ClassType = CT_PCI;
	Desc = "66Mhz Capable";
	break;
    case 0x0F:	DevInfo->ClassType = CT_AGP;		break;
    case 0x10:	
	DevInfo->ClassType = CT_AGP;
	Desc = "2X";
	break;
    case 0x11:	
	DevInfo->ClassType = CT_AGP;
	Desc = "4X";
	break;
    case 0x12:	DevInfo->ClassType = CT_PCIX;		break;
    case 0xA0:	Desc = "PC-98/C20";			break;
    case 0xA1:	Desc = "PC-98/C24";			break;
    case 0xA2:	Desc = "PC-98/E";			break;
    case 0xA3:	Desc = "PC-98/Local Bus";		break;
    case 0xA4:	Desc = "PC-98/Card";			break;
     default:
	SImsg(SIM_DBG, "DMI SLOT: Unknown Slot Type=0x%02X", Slot->Type);
    }

    DevInfo->Unit = Unit++;
    snprintf(Buff, sizeof(Buff), "%s%d", (Dev) ? Dev : "slot", DevInfo->Unit);
    DevInfo->Name = strdup(Buff);

    if (Desc)
	DevInfo->ModelDesc = strdup(Desc);

    if (Slot->Usage == 0x03) 
	AddDesc(lp, "Status", strdup("Available"), 0);
    else if (Slot->Usage == 0x04) 
	AddDesc(lp, "Status", strdup("In Use"), 0);

    if (Design)
	AddDesc(lp, "Designation", strdup(Design), 0);

    if (cp = GetArrayEntry(BusWidths, Slot->BusWidth))
	AddDesc(lp, "Data Bus Width", strdup(cp), 0);
    if (cp = GetArrayEntry(Lengths, Slot->Length))
	AddDesc(lp, "Slot Length", strdup(cp), 0);

    if (Slot->v50)
	AddDesc(lp, s, "5.0v", 0);
    if (Slot->v33)
	AddDesc(lp, s, "3.3v", 0);
    if (Slot->SlotShared)
	AddDesc(lp, s, "Slot Opening Shared", 0);
    if (Slot->SupPCCard16)
	AddDesc(lp, s, "PC Card-16", 0);
    if (Slot->SupCardBus)
	AddDesc(lp, s, "PC Card CardBus", 0);
    if (Slot->SupZoomVid)
	AddDesc(lp, s, "PC Card Zoom Video", 0);
    if (Slot->ModemRingRes)
	AddDesc(lp, s, "Modem Ring Resume", 0);
    if (Hdr->Length >= 0x0C) {
	if (Slot->SupPME)
	    AddDesc(lp, s, "Power Management Enable", 0);
	if (Slot->SupSMBus)
	    AddDesc(lp, s, "SMBus", 0);
    }

    return DevInfo;
}
