#
# $Id: uiDataColumn.pm,v 1.7 2002/03/16 01:33:55 mcooper Exp $
#
# UI Data Columns functions
#

package uiDataColumn;

require 5.003;

require Exporter;

use mcDebug;
use mcGetSize;
use uiMisc;
use uiGlobals;
use uiIcon;

my $Format;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   DataColumnType DataColumnAdd
		  );
}

#
# Return the type of column
#
sub DataColumnType {
    my $Path = shift;
    my $Col = shift;

    if (!$Path) {
	return;
    }

    my $Class = uiClass::ClassGet($Path);
    if (!$Class) {
	Debug("DataColumnType: Cannot determine class from Path=<%s>", $Path);
	return;
    }

    my @Headers = @{ $Class->{Headers} };
    if ($Col > $#Headers) {
	Debug("DataColumnType: Requested column # (%d) is too large (max %d) Path=<%s>",
	      $Col, $#Headers, $Path);
	return;
    }

##    Debug("TYPE: Class=<%s> Col=%d Type=<%s>", $Class->{Name}, $Col,
##	  $Headers[$Col]->{Type});

    return $Headers[$Col]->{Type};
}

#
# Get the Data in columns ready format
#
sub DataGetColumns {
    my $Args = shift;
    my $Entry = $Args->{Entry};
    my $Name = $Args->{Name};
    my $Path = $Args->{Path};
    my $Class = $Args->{Class};
    my $Widget = $Args->{Widget};

    if (!$Entry) {
	return;
    }

    if (Eq($Path, $Class->{Name})) {
	Debug("DataGetColumns: Skipping Path==Class <%s>=<%s>",
	      $Path, $Class->{Name});
	return;
    }

    my @a;
    if ($Entry =~ /mcDevInfo/) {
	# Special handling of "Total Disk" entry
	if (lc($Path) =~ /total disk/) {
#	    if (length($Entry->ModelDesc)) { push(@a, $Entry->ModelDesc); }
	    return;
	} else {
	    # Order must match @HardwareHeaders
	    my $image;
	    if (my $icon = GetIcon($Args)) {
		$image = $Widget->Getimage($icon);
	    }
	    push(@a, $image);
	    push(@a, $Name);
	    push(@a, $Entry->Vendor || '');
	    push(@a, $Entry->Model || '');
	    push(@a, $Entry->Type || '');
	    push(@a, $Entry->ClassType || '');
	    push(@a, $Entry->Speed || '');
	    push(@a, $Entry->Capacity || '');
	}
    } elsif ($Entry =~ /mcPartInfo/) {
	#
	# We can be called both by class Hardware for display as a subitem
	# as well as by class Partition, both of which use different formats.
	#
	if (Eq($Class->{Name}, 'hardware') || Eq($Class->{Name}, 'device')) {
	    # Order must match @HardwareHeaders
	    push(@a, '');				# Icon
	    push(@a, $Name);
	    push(@a, lc($Entry->UsageStatus) || '');	# Vendor
	    push(@a, $Entry->MntName || '');		# Model
	    push(@a, $Entry->Type || '');
	    push(@a, '');				# Class
	    push(@a, '');				# Speed
	    push(@a, GetSizeStr($Entry->Size, $MBYTES) || '');
	} else {
	    # Order must match @PartHeaders
	    push(@a, $Name);
	    push(@a, GetSizeStr($Entry->Size, $MBYTES) || '');
	    push(@a, GetSizeStr($Entry->AmtUsed, $MBYTES) || '');
	    push(@a, $Entry->Type || '');
	    push(@a, $Entry->MntName || '');
	}
    } elsif ($Entry =~ /mcPrinterInfo/) {
	my $image;
	if (my $icon = GetIcon({Path=>lc($Class->{Name})})) {
	    $image = $Widget->Getimage($icon);
	}
	push(@a, $image);
	push(@a, $Name);
	push(@a, join(", ", $Entry->Aliases));
	push(@a, $Entry->Server);
	push(@a, join(", ", $Entry->Langs));
	push(@a, $Entry->Descript);
    } elsif ($Entry =~ /mcCacheInfo/) {
	# Order must match @HardwareHeaders
	my $image = $Widget->Getimage($Icons->{cache});
	push(@a, $image);
	push(@a, $Name);
	push(@a, '');					# Vendor
	push(@a, '');					# Model
	push(@a, $Entry->Type || '');
	push(@a, 'cache');				# Class
	push(@a, '');					# Speed
	push(@a, GetSizeStr($Entry->Size, $KBYTES));
    } elsif ($Entry =~ /mcSoftInfo/) {
	# Order must match @SoftwareHeaders
	push(@a, $Name);
	push(@a, $Entry->Version || '');
	push(@a, $Entry->Revision || '');
	push(@a, GetSizeStr($Entry->DiskUsage, $BYTES) || '');
	push(@a, $Entry->Description || '');
    } elsif ($Path =~ /^SysConf/) {
	if ($Entry !~ /HASH/) {
	    return;
	}
	if (HasMember($Entry, 'Desc') && $Entry->Desc) {
	    push(@a, $Entry->Desc);
	} else {
	    push(@a, '');
	}
	if (HasMember($Entry, 'Name') && $Entry->Name) {
	    push(@a, $Entry->Name);
	} else {
	    push(@a, '');
	}
	if (HasMember($Entry, 'Value')) {
	    push(@a, $Entry->Value);
	} else {
	    push(@a, '');
	}
    } elsif ($Path =~ /^Kernel/ || $Path =~ /^General/) {
	if ($Entry !~ /HASH/) {
	    return;
	}
	if (HasMember($Entry, 'Desc') && $Entry->Desc) {
	    push(@a, $Entry->Desc);
	} else {
	    push(@a, '');
	}
	if (HasMember($Entry, 'Value')) {
	    push(@a, $Entry->Value);
	} else {
	    push(@a, '');
	}
    } elsif (Eq($Class->{Name}, 'Hardware')) {
	# Order must match @HardwareHeaders
	my $image;
	if (my $icon = GetIcon($Args)) {
	    $image = $Widget->Getimage($icon);
	}
	push(@a, $image || '');
	push(@a, '');
	push(@a, $Name);			# Vendor
	push(@a, '');				# Model
	push(@a, '');
	push(@a, '');				# Class
	push(@a, '');				# Speed
	push(@a, '');				# Size
    }

    return @a;
}

#
# Add an entry to a data column
#
sub DataColumnAdd {
    my $Args = shift;
    my $Entry = $Args->{Entry};
    my $Name = $Args->{Name};
    my $Path = $Args->{Path};
    my $Class = $Args->{Class};
    my $Widget = $Args->{Widget};
    my $Style = $Args->{Style};
    my $HeaderStyle = $Args->{HeaderStyle};
    my @Entries = @{ $Args->{Entries} };

    if ($#Entries < 0) {
	@Entries = DataGetColumns({Name=>$Name, Entry=>$Entry,
				   Widget=>$Widget,
				   Class=>$Class,
				   Path=>$Path});
    }
    if ($#Entries < 0) {
	Debug("DataColumnAdd: No Entries for Path=<%s>", $Path);
	return -1;
    }

    #
    # Hardware entries with nothing in column 1 (DEVICE) mean it's a 
    # header.
    #
    if ($HeaderStyle && Eq($Class->{Name}, 'hardware') &&
	!length($Entries[1])) {
	$Style = $HeaderStyle;
    }

    # Now add each column value
    for (my $i = 0; $i <= $#Entries; ++$i) {
	if (!$Widget->headerExists($i)) {
	    Error("Header doesn't exist for column %d: Path=<%s>",
		  $i, $Path);
	    next;
	}

	if (my $type = DataColumnType($Path, $i)) {
	    if ($type eq 'text') {
		$Widget->itemCreate($Path, $i,
				    -itemtype=>$type,
				    -style=>$Style->{lc($type)},
				    -text=>$Entries[$i]);
	    } elsif ($type eq 'image') {
		if ($Entries[$i] && length($Entries[$i])) {
		    $Widget->itemCreate($Path, $i,
					-itemtype=>$type,
					-style=>$Style->{lc($type)},
					-image=>$Entries[$i]);
		}
	    }
	}
    }

    return 0;
}

1;
