/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.43 $";
#endif

/*
 * Class Info
 */
#include "defs.h"

/*
 * Generic headers
 */
#define HeaderNamesNorm	{ "DESCRIPTION", "VALUE", NULL }
#define HeaderNamesAll	{ "DESCRIPTION", "KEY", "VALUE", NULL }

/*
 * Header format definetions
 */
HeaderFormat_t HeadersNorm = {
    HeaderNamesNorm,	2,
    { 0, 0 },		2,
    { 0, 0 },		2,
};
HeaderFormat_t HeadersAll = {
    HeaderNamesAll,	3,
    { 0, 0, 0 },	3,
    { 0, 0, 0 },	3,
};
HeaderFormat_t PartHeader = {
    { "PART", "CAPACITY", "USED", "TYPE", "USAGE", NULL },	5,
    { 10, 10, 10, 10, 0 },	5,
    { 13, 13, 13, 13, 46 },	5,
};
HeaderFormat_t HardwareHeader = {
    { "DEVICE", "VENDOR", "MODEL", "TYPE", "CLASS", "SPEED", "CAPACITY",
      NULL },				7,
    { 12, 8, 19, 8, 10, 8, 9 },		7,
    { 16, 11, 25, 11, 14, 11, 12 },	7,
};
HeaderFormat_t SoftwareHeader = {
    { "PACKAGE", "VERSION", "DESCRIPTION", NULL },	3,
    { 20, 15, 0 },	3,
    { 30, 12, 58 },	3,
};
HeaderFormat_t PrinterHeader = {
    { "QUEUE", "SERVER", "DESCRIPTION", NULL },	3,
    { 20, 15, 0 },	3,
    { 30, 12, 58 },	3,
};

/*
 * Class information
 */
ClassInfo_t ClassInfo[] = {
    { CN_ALL,			{ NULL }, 
      NULL,			NULL,
      "All classes of information",
      NULL,
      NULL, NULL,
      FALSE },
    { CN_GENERAL,		{ NULL }, 
      GeneralShow,		GeneralList,
      "General Information",
      NULL,
      &HeadersNorm, &HeadersAll,
      TRUE },
    { CN_KERNEL,		{ NULL },
      KernelShow,		KernelList,
      "Kernel Information",
      NULL,
      &HeadersNorm, &HeadersAll,
      FALSE },
    { CN_SYSCONF,		{ NULL },
      SysConfShow,		SysConfList,
      "SysConf Information",
      NULL,
      &HeadersNorm, &HeadersAll,
      FALSE },
    { CN_PRINTER,		{ NULL },
      ShowPrinter,		ListPrinter,
      "Printer Information",
      NULL,
      &PrinterHeader, &PrinterHeader,
      FALSE },
    { CN_PARTITION,		{ NULL },
      ShowPartition,		ListPartition,
      "Partition Information",
      NULL,
      &PartHeader, &PartHeader,
      FALSE },
    { CN_DEVICE,		{ CN_HARDWARE, NULL },
      DeviceShow,		DeviceList,
      "Hardware Information",
      NULL,
      &HardwareHeader, &HardwareHeader,
      TRUE },
    { CN_SOFTINFO,		{ NULL },
      ShowSoftInfo,		ListSoftInfo,
      "Software Information",
      NULL,
      &SoftwareHeader, &SoftwareHeader,
      FALSE },
    { 0 },
};

/*
 * List class names.
 */
extern void ClassList()
{
    register ClassInfo_t       *ClassPtr;
    static char			Buff[BUFSIZ];
    char		       *Name;
    register char	      **cpp;

    if (!VL_TERSE) {
	SImsg(SIM_INFO, 
"The following values may be specified with the `-class' option:\n");
	SImsg(SIM_INFO, "%-20s %s\n", "VALUE", "DESCRIPTION");
    }

    for (ClassPtr = &ClassInfo[0]; ClassPtr->Name; ++ClassPtr) {
	if (ClassPtr->Aliases) {
	    snprintf(Buff, sizeof(Buff), ClassPtr->Name);
	    for (cpp = ClassPtr->Aliases; cpp && *cpp; ++cpp) {
		Name = &Buff[strlen(Buff)];
		snprintf(Name, sizeof(Buff)-strlen(*cpp)-3, ",%s", *cpp);
	    }
	    Name = Buff;
	} else {
	    Name = ClassPtr->Name;
	}
	SImsg(SIM_INFO, "%-20s %s\n", Name, ClassPtr->Label);
    }
}

/*
 * Lookup a class named Name.
 */
extern ClassInfo_t *ClassGetByName(Name)
    char		       *Name;
{
    register ClassInfo_t       *ClassPtr;
    register char	      **cpp;

    for (ClassPtr = &ClassInfo[0]; ClassPtr->Name; ++ClassPtr) {
	if (EQ(Name, ClassPtr->Name))
	    return(ClassPtr);
	for (cpp = ClassPtr->Aliases; cpp && *cpp; ++cpp)
	    if (EQ(Name, *cpp))
		return(ClassPtr);
	    
    }

    return((ClassInfo_t *) NULL);
}

/*
 * Set class information
 */
extern void ClassSetInfo(Names)
    char		       *Names;
{
    register ClassInfo_t       *ClassPtr;
    register char	       *cp;
    char		       *String;

    if (!Names)
	return;

    if (EQ(Names, CN_ALL))
	/*
	 * Enable all classes
	 */
	for (ClassPtr = &ClassInfo[0]; ClassPtr->Name; ++ClassPtr)
	    ClassPtr->Enabled = TRUE;
    else {
	/*
	 * Enable a specific list of classes.
	 * The Names variable is a `,' seperated list
	 * of class names.
	 */
	String = (char *) strdup(Names);
	for (cp = (char *) strtok(String, ","); cp;
	     cp = (char *) strtok((char *)NULL, ",")) {
	    ClassPtr = ClassGetByName(cp);
	    if (!ClassPtr) {
		SImsg(SIM_GERR, "The class name `%s' is invalid.", cp);
		ClassList();
		exit(1);
	    }
	    ClassPtr->UserEnabled = TRUE;
	}
	(void) free(String);

	/* Since user specified class, disable defaults */
	for (ClassPtr = &ClassInfo[0]; ClassPtr->Name; ++ClassPtr)
	    ClassPtr->Enabled = FALSE;
    }
}

/*
 * Call each enabled show function
 */
extern int ClassCall(NameStr)
    char		       *NameStr;
{
    ClassInfo_t		       *ClassPtr;
    char		      **Argv;
    int				Argc;
    int				Status;
    int				ErrCount = 0;
    int				First = 1;

    Argc = StrToArgv(NameStr, ",", &Argv, NULL, 0);

    for (ClassPtr = &ClassInfo[0]; ClassPtr->Name; ++ClassPtr)
	if ((ClassPtr->Enabled || ClassPtr->UserEnabled) && ClassPtr->Show) {
	    if (First) {
		if (EncodeType == OE_HTML)
		    HtmlStart();
		First = 0;
	    }

	    Status = (*ClassPtr->Show)(ClassPtr,
				       (Argc > 0) ? Argv : (char **) NULL);

	    if (Status == CLASS_UNSUPPORTED && ClassPtr->UserEnabled) {
		SImsg(SIM_CERR, "Class `%s' is unsupported on this system.",
		      ClassPtr->Name);
		++ErrCount;
	    }
	}

    if (!First && EncodeType == OE_HTML)
	HtmlEnd();

    return (ErrCount == 0) ? 0 : -1;
}

/*
 * Call each Class List function
 */
extern void ClassCallList()
{
    register ClassInfo_t       *ClassPtr;

    for (ClassPtr = &ClassInfo[0]; ClassPtr->Name; ++ClassPtr)
	if (ClassPtr->List)
	    (*ClassPtr->List)();
}
