/*
 * Copyright (c) 1992-2003 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.7 $";
#endif

/*
 * Show Printer related functions.
 */

#include "defs.h"
#include "html.h"

/*
 * Printer Protocol names - must match PP_* in mcprinter.h
 */
static char *PrinterProtoList[] = PRINTER_PROTO_LIST;

/*
 * Return name of Printer Protocol
 */
extern char *PrinterProto(Proto)
     int		        Proto;
{
    if (Proto > sizeof(PrinterProtoList)/sizeof(char *)) {
	SImsg(SIM_DBG, "PrinterProto: Proto=%d is not a legal value.", Proto);
	return (char *) NULL;
    }

    return PrinterProtoList[Proto];
}

/*
 * List valid arguments for Printer class. 
 */
extern void ListPrinter()
{
    SImsg(SIM_INFO, 
	  "\n\nTo show specific printer use `-class printer -show prt1,prt2...'.\n\n");
}

/*
 * Show printer details in OE_TEXT or OE_HTML
 */
static void ShowPrinterDetailsText(Printer, OffSet)
     Printer_t		       *Printer;
     int			OffSet;
{ 
    char		      **cpp;
    static char			Buff[256];
    Desc_t		       *Desc;

    if (!VL_DESC && !VL_CONFIG)
	return;

#define CHK(a)	( a && *(a) )
    ShowLabelStart();

    ShowLabel("Queue Name", Printer->Queue, OffSet);

    if (CHK(Printer->Aliases)) {
	Buff[0] = CNULL;
	for (cpp = Printer->Aliases; cpp && *cpp; ++cpp) {
	    if (Buff[0])
		StrAppend(Buff, sizeof(Buff), ", ");
	    StrAppend(Buff, sizeof(Buff), *cpp);
	}
	ShowLabel("Queue Aliases", Buff, OffSet);
    }

    if (CHK(Printer->Langs)) {
	Buff[0] = CNULL;
	for (cpp = Printer->Langs; cpp && *cpp; ++cpp) {
	    if (Buff[0])
		StrAppend(Buff, sizeof(Buff), ", ");
	    StrAppend(Buff, sizeof(Buff), *cpp);
	}
	ShowLabel("Printer Languages", Buff, OffSet);
    }

    if (CHK(Printer->Desc))
	ShowLabel("Description", Printer->Desc, OffSet);
    if (CHK(Printer->Vendor))
	ShowLabel("Vendor", Printer->Vendor, OffSet);
    if (CHK(Printer->Model))
	ShowLabel("Model", Printer->Model, OffSet);
    if (Printer->Proto)
	ShowLabel("Protocol", PrinterProto(Printer->Proto), OffSet);
    if (CHK(Printer->Device))
	ShowLabel("Device", Printer->Device, OffSet);
    if (CHK(Printer->Server))
	ShowLabel("Server", Printer->Server, OffSet);
    if (CHK(Printer->RemoteQueue))
	ShowLabel("Remote Queue", Printer->RemoteQueue, OffSet);
    if (CHK(Printer->SpoolDir))
	ShowLabel("Spool Directory", Printer->SpoolDir, OffSet);

    if (Printer->MaxJobSize >= 0)
	ShowLabel("Max Job Size",
		  (Printer->MaxJobSize == 0) ? "unlimited" : GetSizeStr((Large_t)Printer->MaxJobSize, KBYTES), OffSet);

    if (Printer->DescList)
	for (Desc = Printer->DescList; Desc; Desc = Desc->Next)
	    ShowLabel(GetDescLabel(Desc), Desc->Desc, OffSet);

    ShowLabelEnd();
#undef CHK
}

/*
 * Switch for Details of Printer
 */
static void ShowPrinterDetails(Printer, OffSet)
     Printer_t		       *Printer;
     int			OffSet;
{
    switch (EncodeType) {
    case OE_HTML:
    case OE_TEXT:
	ShowPrinterDetailsText(Printer, OffSet);
	break;
    }
}

/*
 * Tree Display of a Printer entry.
 */
static void ShowPrinterTree(Printer, OffSet)
     Printer_t		       *Printer;
     int			OffSet;
{
    char		      **cpp;
    static char			Buff[256];

    if (!Printer)
	return;

    snprintf(Buff, sizeof(Buff), "%-15s", ShowEncodeName(Printer->Queue));

    if (Printer->Vendor || Printer->Model)
	StrAppend(Buff, sizeof(Buff), " is a");
    if (Printer->Vendor) {
	StrAppend(Buff, sizeof(Buff), " ");
	StrAppend(Buff, sizeof(Buff), Printer->Vendor);
    }
    if (Printer->Model) {
	StrAppend(Buff, sizeof(Buff), " ");
	StrAppend(Buff, sizeof(Buff), Printer->Model);
    }

    if (Printer->Desc) {
	StrAppend(Buff, sizeof(Buff), " \"");
	StrAppend(Buff, sizeof(Buff), Printer->Desc);
	StrAppend(Buff, sizeof(Buff), "\"");
    }

    if (Printer->Proto) {
	StrAppend(Buff, sizeof(Buff), " ");
	StrAppend(Buff, sizeof(Buff), PrinterProto(Printer->Proto));
    }

    if (Printer->Langs && Printer->Langs[0]) {
	StrAppend(Buff, sizeof(Buff), " ");
	StrAppend(Buff, sizeof(Buff), Printer->Langs[0]);
    }

    if (Printer->Device) {
	StrAppend(Buff, sizeof(Buff), " on ");
	StrAppend(Buff, sizeof(Buff), Printer->Device);
    }

    if (Printer->Server) {
	StrAppend(Buff, sizeof(Buff), " on ");
	StrAppend(Buff, sizeof(Buff), Printer->Server);
    }

    ShowLine(OffSet, Buff);

    ShowPrinterDetails(Printer, OffSet);
}

/*
 * Column Display of a Printer entry as OE_TEXT
 */
extern void ShowPrinterColumnsText(Printer, OffSet)
     Printer_t		       *Printer;
     int			OffSet;
{
    char		      **cpp;
    static char		       *FormatStr;
    char		      **Format;
    ClassInfo_t		       *Class;

    if (!Printer)
	return;

    if (!FormatStr) {
	Class = ClassGetByName(CN_PRINTER);
	if (!Class) {
	    SImsg(SIM_DBG, "ShowPrinterColumnsText: Cannot find %s class",
		  CN_PRINTER);
	    return;
	}
	Format = ColGetFmt(Class->Headers);
	if (!Format)
	    return;

	FormatStr = ColGetFmtStr(Format);
	if (!FormatStr)
	    return;
    }

    SImsg(SIM_INFO, FormatStr,
	  CPRT(Printer->Queue),
	  CPRT(Printer->Server),
	  CPRT(Printer->Desc));
}

/*
 * Column Display of a Printer entry as OE_HTML
 */
extern void ShowPrinterColumnsHtml(Printer, OffSet)
     Printer_t		       *Printer;
     int			OffSet;
{
    char		       *DataArgv[3];
    static int			Counter;
    char		        Options[128];

    if (!Printer)
	return;

    DataArgv[0] = PRTS(Printer->Queue);
    DataArgv[1] = PRTS(Printer->Server);
    DataArgv[2] = PRTS(Printer->Desc);

    /* Rows alternate colors */
    if (Counter++ % 2)
	snprintf(Options, sizeof(Options), "bgcolor=\"%s\"",
		 HTML_BGLINE_1A);
    else
	snprintf(Options, sizeof(Options), "bgcolor=\"%s\"",
		 HTML_BGLINE_1B);

    HtmlTableData(DataArgv, sizeof(DataArgv)/sizeof(char *), NULL, Options);
}

/*
 * Show FT_REPORT for Printer
 */
static void ShowPrinterReport(Printer, OffSet)
     Printer_t		       *Printer;
     int			OffSet;
{
    static char		       *RptData[11];
    Desc_t		       *Desc;
    char		       *String1 = NULL;
    char		       *String2 = NULL;
    char		       *cp;

    (void) memset(RptData, CNULL, sizeof(RptData));
    if (Printer->Aliases)
	if (ArgvToStr(&String1, Printer->Aliases, 0, ",", 0) == 0)
	    RptData[0] = PRTS(String1);
    RptData[1] = PRTS(Printer->Desc);
    RptData[2] = PRTS(Printer->Vendor);
    RptData[3] = PRTS(Printer->Model);
    if (Printer->Proto)
	RptData[4] = PRTS(PrinterProto(Printer->Proto));
    RptData[5] = PRTS(Printer->Device);
    RptData[6] = PRTS(Printer->Server);
    RptData[7] = PRTS(Printer->RemoteQueue);
    if (Printer->Langs)
	if (ArgvToStr(&String2, Printer->Langs, 0, ",", 0) == 0)
	    RptData[8] = PRTS(String2);
    RptData[9] = PRTS(Printer->SpoolDir);
    if (Printer->MaxJobSize >= 0)
	RptData[10] = (Printer->MaxJobSize == 0) ? "unlimited" : 
	    GetSizeStr((Large_t)Printer->MaxJobSize, KBYTES);

    Report(CN_PRINTER, R_NAME, PRTS(Printer->Queue), RptData,
	   sizeof(RptData) / sizeof(char *));

   /*
     * Report description info
     */
    for (Desc = Printer->DescList; Desc; Desc = Desc->Next) {
	RptData[0] = PRTS(Desc->Label);
	RptData[1] = PRTS(Desc->Desc);
	Report(CN_PRINTER, R_DESC, PRTS(Printer->Queue), RptData, 2);
    }
}

/*
 * Display Printer info
 */
extern int ShowPrinter(MyClass, Names)
    ClassInfo_t		       *MyClass;
    char		      **Names;
{
    static Printer_t 	       *Root = NULL;
    static MCSIquery_t		Query;
    Printer_t		       *Printer;

    (void) memset(&Query, CNULL, sizeof(Query));
    Query.Op = MCSIOP_CREATE;
    Query.Cmd = MCSI_PRINTER;
    if (mcSysInfo(&Query) != 0)
	return CLASS_UNSUPPORTED;

    Root = (Printer_t *) Query.Out;

    if (!Root || !Root->Queue)
	return CLASS_UNSUPPORTED;

    ShowStart(MyClass, (int *)NULL);

    /*
     * Traverse list
     */
    for (Printer = Root; Printer; Printer = Printer->Next) {
	if (!Printer->Queue)
	    continue;
	switch (FormatType) {
	case FT_TREE:
	    ShowPrinterTree(Printer, 0);
	    break;
	case FT_COLUMNS:
	    switch (EncodeType) {
	    case OE_TEXT:
		ShowPrinterColumnsText(Printer, 0);
		ShowPrinterDetails(Printer, 0);
		break;
	    case OE_HTML:
		ShowPrinterColumnsHtml(Printer, 0);
		ShowPrinterDetails(Printer, 0);
		break;
	    }
	    break;
	case FT_REPORT:
	    ShowPrinterReport(Printer, 0);
	    break;
	}
    }

    ShowEnd(MyClass);

    return 0;
}
